#!/usr/bin/perl
#
# Test threads over multi-folders.
#

use Test;
use strict;
use warnings;

use lib qw(. t);

use Tools;
#use Mail::Box::Manager;

BEGIN {plan tests => 1}

warn "   * Multi-folder threads status ALPHA (not tested)\n";
ok(1);
